/*---------------------------------------------------------*/
/* REXX program to perform sockets interface calls         */
/* Server module which interfaces with RAFSRVM             */
/*---------------------------------------------------------*/

svrq = "rafqm"
mySleep = 1
sendEof = "no"

mproto = "SOCK_STREAM"
mdomain = "AF_INET"
mport   = 2300
maddr   = "INADDR_ANY"


/* init stuff */

PARSE ARG xport other

IF (xport <> "") THEN
   mport = xport

rc = RxFuncAdd("SockLoadFuncs", "rxSock", "SockLoadFuncs")
rc = SockLoadFuncs()

serv2.!family = mdomain
serv2.!port   = mport
serv2.!addr   = maddr

myq   = RXQUEUE("Create")              /*  get a queuename for me */
oldq  = RXQUEUE('Set', svrq)           /*  use the new queue    */

/* create a socket    */

s1 = SockSocket(mdomain, mproto, 0)

SAY SockGetHostId()

SAY "create socket returned "s1
IF (s1 < 0) THEN
  DO
     SockPSock_Errno("SockSocket()")
     rc = SockClose(s1)
     RETURN
  END

/* bind socket to any address  */

rc = SockBind(s1, "serv2.!")
SAY "socket bind returned "rc
IF (rc < 0) THEN
  DO
     SockPSock_Errno("SockBind()")
     RETURN
  END

rc = SockGetSockName(s1, "serv2.!")
SAY "socket port is "serv2.!port

/* establish listen on the socket   */

srvMore = 1
DO WHILE srvMore == 1
  rc = SockListen(s1, 8)
  SAY "listen returned "rc

  IF (rc < 0) THEN
    DO
       SockPSock_Errno("SockListen()")
       rc = SockClose(s1)
       RETURN
    END

/* process requests                 */

  Call xprocess

END

/* close socket       */

  call RXQUEUE "Delete", myq
  call RXQUEUE "Set", oldq


  rc = SockClose(s1)

/* terminate and exit */

Return

/*-----------------------------------------------*/
/* procedure to handle process mode              */
/*-----------------------------------------------*/

xProcess:

  sClient = SockAccept(s1)
  SAY "accepted socket "sClient
  bMore = 1

  IF (sClient < 0) THEN
    RETURN
  
  rc = SockRecv(sClient, "inMsg", 1024)
  SAY "rc="rc "data: "inMsg
  
  IF (rc > 0) THEN
    DO
      IF (inMsg == "serverquit") THEN
        DO
          srvMore = 0
          rc = SockClose(sClient)
          RETURN
        END

      Call RXQUEUE 'Set', svrq               /*  use the new queue    */
      PUSH time() " " myq " " inMsg
      call RXQUEUE "Set", myq                /*  switch to my queue     */

      DO WHILE QUEUED() == 0
        Call SysSleep mySleep
      END

      DO WHILE QUEUED() > 0
        parse pull qElement
        rc = SockSend(sClient, qElement)
        rc2 = SockRecv(sClient, "tempbuf", 32)
      END

SAY "out of loop"
      rc = SockSend(sClient, sendEof)

    END

  rc = SockClose(sClient)
  RETURN


